import { isJSONObject } from '../../helpers/json.js';
import { isRecordOfStringArray } from './types.js';

/**
 * Builds the default message for {@link ScalewayError}.
 *
 * @param status - The response code
 * @param body - The response body
 * @returns The error message
 *
 * @internal
 */
const buildDefaultMessage = (status, body) => {
  const message = [`http error ${status}`];
  if (typeof body === 'string') {
    message.push(body);
  } else if (isJSONObject(body)) {
    if (typeof body.resource === 'string') {
      message.push(`resource ${body.resource}`);
    }
    if (typeof body.message === 'string') {
      message.push(body.message);
    }
    if (body.fields && isRecordOfStringArray(body.fields)) {
      message.push(Object.entries(body.fields).map(([name, list]) => `${name} (${list.join(', ')})`).join(', '));
    }
  }
  return message.join(': ');
};

/**
 * Scaleway error.
 *
 * @public
 */
class ScalewayError extends Error {
  /** The message originating from the payload. */

  constructor( /** The response status. */
  status, /** The response payload. */
  body, /** The augmented message. */
  message = buildDefaultMessage(status, body)) {
    super(message); // 'Error' breaks prototype chain here
    this.status = status;
    this.body = body;
    this.message = message;
    this.name = 'ScalewayError';
    this.rawMessage = typeof body === 'object' && typeof body.message === 'string' ? body.message : undefined;
    Object.setPrototypeOf(this, new.target.prototype); // restore prototype chain
  }
  static fromJSON(status, obj) {
    return new ScalewayError(status, obj);
  }
  toString() {
    return `${this.name}: ${this.message}`;
  }
}

export { ScalewayError };
