/**
 * Converts a string to PascalCase.
 *
 * @param str - The input string
 * @returns The string in PascalCase
 *
 * @internal
 */
const toPascalCase = str => str.replace(/\w+/g, word => `${word[0].toUpperCase()}${word.slice(1).toLowerCase()}`);

/**
 * Converts a Headers entry to string.
 *
 * @param entry - The header entry as a string tuple
 * @returns A serialized string
 *
 * @internal
 */
const serializeHeadersEntry = ([name, value]) => `${toPascalCase(name)}: ${value}`;

/**
 * Converts Headers to safe to log strings (with obfuscated auth secrets).
 *
 * @param headers - The Headers
 * @returns Serialized headers strings
 *
 * @internal
 */
const serializeHeaders = headers => Array.from(headers.entries(), serializeHeadersEntry);

/**
 * Dumps a Request into a readable string.
 *
 * @param request - The request
 * @returns The readable string
 *
 * @internal
 */
const dumpRequest = async request => [`${request.method.toUpperCase()}: ${request.url}`, ...serializeHeaders(request.headers), await request.clone().text()].join('\r\n');

/**
 * Dumps a Response into a readable string.
 *
 * @param response - The response
 * @returns The readable string
 *
 * @internal
 */
const dumpResponse = async response => [`HTTP ${response.status} ${response.ok ? 'OK' : 'NOK'}`, ...serializeHeaders(response.headers), await response.clone().text()].join('\r\n');

export { dumpRequest, dumpResponse };
